<?php

namespace Ems\Services;

use Doctrine\ORM\EntityManager;
use Ems\Repository\{ ParceiroNegocioRepository, PeriodoRepository };
use Ems\Services\ParamOrgService;
use Ems\Util\Utils;
use Util\Repositories;
use Util\{ Environment, NovoCodigo };

class ParceiroNegocioService {
    /** @var Environment */
    protected $environment;

    /** @var ParceiroNegocioRepository */
    protected $parceiroNegocioRepository;

    public function __construct(
        EntityManager               $entityManager,
        Environment                 $environment,
        ParamOrgService             $paramOrgService,
        ParceiroNegocioRepository   $parceiroNegocioRepository,
        PeriodoRepository           $periodoRepository
    ) {
        $this->entityManager                = $entityManager;
        $this->environment                  = $environment;
        $this->paramOrgService              = $paramOrgService;
        $this->parceiroNegocioRepository    = $parceiroNegocioRepository;
        $this->periodoRepository            = $periodoRepository;
    }
    
    protected const CDFORMACOMU_EMAIL = '05';
    
    public function getParceirosNegocioPerfilAcesso(string $perfilAcesso = null) {
        $nrorg = (int) $this->environment->getNrorg();
        
        $perfilAcesso = (int) $this->paramOrgService->getPerfilAcessoOrg($perfilAcesso);
        
        $parcNegocioPerfilAcesso = [];
        
        if (!empty($perfilAcesso) || $perfilAcesso == '0') {
            $parcNegocioPerfilAcesso = $this->parceiroNegocioRepository->getParceiroNegocioByPerfilAcesso($nrorg, $perfilAcesso);
        }
        
        return $parcNegocioPerfilAcesso;
    }
    
    public function getEmailsPorPerfilAcesso() {
        $nrorg = $this->environment->getNrorg();
        $nrParcsNegocio = $this->getParceirosNegocioPerfilAcesso();
        
        $comunicaParcs = $this->entityManager->getRepository(Repositories::COMUNICAPARC)->findBy(array('nrorg' => $nrorg, 'nrparcnegocio' => $nrParcsNegocio, 'cdformacomu' => self::CDFORMACOMU_EMAIL));
        
        $comunicaParcsEmails = array_map(function($comunicaParc) { return $comunicaParc->getCdcomunicaparc(); }, $comunicaParcs);
        return $comunicaParcsEmails;
    }
    
    public function getEmailParcNegocio($nrparnegocio) {
        $nrorg = $this->environment->getNrorg();
        $comunicaParc = $this->entityManager->getRepository(Repositories::COMUNICAPARC)->findOneBy(array('nrorg' => $nrorg, 'nrparcnegocio' => $nrparnegocio, 'cdformacomu' => self::CDFORMACOMU_EMAIL));
        
        if (!empty($comunicaParc)) {
            return $comunicaParc->getCdcomunicaparc();
        }
        
        return false;
        
    }
    
    public function getComunicacaoParc($nrorg, $nrvinculom) {
         if (empty($nrorg)) {
            $nrorg = $this->environment->getNrorg();
        }
        
        $comunicaParc = $this->parceiroNegocioRepository->getComunicacaoParc($nrorg, $nrvinculom);
        
        return $comunicaParc;
    }
    
    public function getContaCorrenteParc($nrorg, $nrvinculom) {
         if (empty($nrorg)) {
            $nrorg = $this->environment->getNrorg();
        }
        
        $comunicaParc = $this->parceiroNegocioRepository->getContaCorrenteParc($nrorg, $nrvinculom);
        
        return $comunicaParc;
    }
    
    public function getEnderecoParc($nrorg, $nrvinculom) {
         if (empty($nrorg)) {
            $nrorg = $this->environment->getNrorg();
        }
        
        $comunicaParc = $this->parceiroNegocioRepository->getEnderecoParc($nrorg, $nrvinculom);
        
        return $comunicaParc;
    }

    /**
     * Cria um novo registro na PARCNEGOCIO com os dados enviados na row.
     *
     * @param array $row Dados do parceiro de negócio.
     *
     * @return int O número do parceiro de negócio criado.
     */
    public function setParceiroNegocio(
        array $row
    ): int {
        $nrOrg = (int) $this->environment->getNrOrg();
        $cdOperador = (string) $this->environment->getCdOperador();

        if (!isset($row['NRPARCNEGOCIO'])) {
            $row['NRPARCNEGOCIO'] = NovoCodigo::geraCodigo($nrOrg, 'PARCNEGOCIO', 1, 12, true, false);
        }

        $row['NRORG']               = $nrOrg;
        $row['CDSEGMER']            = $row['CDSEGMER'] ?? null;
        $row['NMPRINCIPALPARC']     = $row['NMPRINCIPALPARC'] ?? null;
        $row['NMSECUNDARIPARC']     = $row['NMSECUNDARIPARC'] ?? null;
        $row['DTNASCIFUNDPARC']     = $row['DTNASCIFUNDPARC'] ?? null;
        $row['NRINSCRICAOPARC']     = $row['NRINSCRICAOPARC'] ?? null;
        $row['IDATIVO']             = $row['IDATIVO'] ?? 'S';
        $row['DTINCLUSAO']          = $row['DTINCLUSAO'] ?? date('d/m/Y');
        $row['DTULTATU']            = $row['DTULTATU'] ?? null;
        $row['NRORGINCLUSAO']       = $row['NRORGINCLUSAO'] ?? $nrOrg;
        $row['CDOPERINCLUSAO']      = $row['CDOPERINCLUSAO'] ?? $cdOperador;
        $row['NRORGULTATU']         = $row['NRORGULTATU'] ?? null;
        $row['CDOPERULTATU']        = $row['CDOPERULTATU'] ?? null;
        $row['CDTIPOPARCPRINCIPAL'] = $row['CDTIPOPARCPRINCIPAL'] ?? null;
        $row['CDTIPOINSCRICAO']     = $row['CDTIPOINSCRICAO'] ?? null;
        $row['IDPESSOAFISICA']      = $row['IDPESSOAFISICA'] ?? null;
        $row['IDINSTITUICAO']       = $row['IDINSTITUICAO'] ?? null;
        $row['DSDEPARTAMENTO']      = $row['DSDEPARTAMENTO'] ?? null;
        $row['IDIDIOMA']            = $row['IDIDIOMA'] ?? null;
        $row['CDATIVCLI']           = $row['CDATIVCLI'] ?? null;
        $row['NRSEQTPATIVCLI']      = $row['NRSEQTPATIVCLI'] ?? null;
        $row['IDPARCFUNDIDO']       = $row['IDPARCFUNDIDO'] ?? null;
        $row['CDOPERRESPONSAVEL']   = $row['CDOPERRESPONSAVEL'] ?? null;
        $row['NRTPCLASSIFICAPARC']  = $row['NRTPCLASSIFICAPARC'] ?? null;
        $row['DSDIRLOGOMARCA']      = $row['DSDIRLOGOMARCA'] ?? null;

        $this->parceiroNegocioRepository->setParceiroNegocio($row);

        return $row['NRPARCNEGOCIO'];
    }

    public function setEnderecoParc(
        array $row
    ): int {
        $nrOrg = (int) $this->environment->getNrOrg();
        $cdOperador = (string) $this->environment->getCdOperador();

        $enderecoRow = [];

        $enderecoRow['NRENDERECOPARC'] =
            $row['NRENDERECOPARC'] ??
            NovoCodigo::geraCodigo($nrOrg, 'ENDERECOPARC', 1, 10, true, false);

        $enderecoRow['NRPARCNEGOCIO']       = $row['NRPARCNEGOCIO'] ?? null;
        $enderecoRow['CDTIPOENDERECO']      = $row['CDTIPOENDERECO'] ?? null;
        $enderecoRow['NRORG']               = $row['NRORG'] ?? $nrOrg;
        $enderecoRow['IDATIVO']             = $row['IDATIVO'] ?? 'S';
        $enderecoRow['DTINCLUSAO']          = $row['DTINCLUSAO'] ?? date('d/m/Y');
        $enderecoRow['DTULTATU']            = $row['DTULTATU'] ?? null;
        $enderecoRow['NRORGINCLUSAO']       = $row['NRORGINCLUSAO'] ?? $nrOrg;
        $enderecoRow['CDOPERINCLUSAO']      = $row['CDOPERINCLUSAO'] ?? $cdOperador;
        $enderecoRow['NRORGULTATU']         = $row['NRORGULTATU'] ?? null;
        $enderecoRow['CDOPERULTATU']        = $row['CDOPERULTATU'] ?? null;
        $enderecoRow['CDPAIS']              = $row['CDPAIS'] ?? null;
        $enderecoRow['SGESTADO']            = $row['SGESTADO'] ?? null;
        $enderecoRow['CDMUNICIPIO']         = $row['CDMUNICIPIO'] ?? null;
        $enderecoRow['CDBAIRRO']            = $row['CDBAIRRO'] ?? null;
        $enderecoRow['CDLOGRADOURO']        = $row['CDLOGRADOURO'] ?? null;
        $enderecoRow['NMBAIRROENDERECO']    = $row['NMBAIRROENDERECO'] ?? null;
        $enderecoRow['DSCOMPLEENDERECO']    = $row['DSCOMPLEENDERECO'] ?? null;
        $enderecoRow['NRCEPENDERECO']       = $row['NRCEPENDERECO'] ?? null;
        $enderecoRow['DSREFERENCIAENDE']    = $row['DSREFERENCIAENDE'] ?? null;
        $enderecoRow['DSENDERECO']          = $row['DSENDERECO'] ?? null;
        $enderecoRow['NRIMOVELENDERECO']    = $row['NRIMOVELENDERECO'] ?? null;
        $enderecoRow['DSLATITUDE']          = $row['DSLATITUDE'] ?? null;
        $enderecoRow['DSLONGITUDE']         = $row['DSLONGITUDE'] ?? null;

        $this->parceiroNegocioRepository->setEnderecoParc($enderecoRow);

        return $enderecoRow['NRENDERECOPARC'];
    }

    public function setContaCorrente(
        array $row
    ): int {
        $nrOrg = (int) $this->environment->getNrOrg();
        $cdOperador = (string) $this->environment->getCdOperador();

        $contaRow = [];

        $contaRow['NRCONTCORRPARC'] =
            $row['NRCONTCORRPARC'] ??
            NovoCodigo::geraCodigo($nrOrg, 'CONTCORRPARC', 1, 10, true, false);

        $contaRow['NRORG']          = $row['NRORG'] ?? $nrOrg;
        $contaRow['NRPARCNEGOCIO']  = $row['NRPARCNEGOCIO'] ?? null;
        $contaRow['CDREDUCONT']     = $row['CDREDUCONT'] ?? null;
        $contaRow['CDBANCO']        = $row['CDBANCO'] ?? null;
        $contaRow['CDAGENCIA']      = $row['CDAGENCIA'] ?? null;
        $contaRow['CDCONTCORR']     = $row['CDCONTCORR'] ?? null;
        $contaRow['NRTPCONTCORR']   = $row['NRTPCONTCORR'] ?? null;
        $contaRow['DTINCLUSAO']     = $row['DTINCLUSAO'] ?? date('d/m/Y');
        $contaRow['DTULTATU']       = $row['DTULTATU'] ?? null;
        $contaRow['NRORGINCLUSAO']  = $row['NRORGINCLUSAO'] ?? $nrOrg;
        $contaRow['CDOPERINCLUSAO'] = $row['CDOPERINCLUSAO'] ?? $cdOperador;
        $contaRow['NRORGULTATU']    = $row['NRORGULTATU'] ?? null;
        $contaRow['CDOPERULTATU']   = $row['CDOPERULTATU'] ?? null;
        $contaRow['IDATIVO']        = $row['IDATIVO'] ?? 'S';

        $this->parceiroNegocioRepository->setContaCorrente($contaRow);

        return $contaRow['NRCONTCORRPARC'];
    }

    public function setComunicaParc(
        array $row
    ): int {
        $nrOrg = (int) $this->environment->getNrOrg();
        $cdOperador = (string) $this->environment->getCdOperador();

        $comuRow = [];

        $comuRow['NRCOMUNICAPARC'] =
            $row['NRCOMUNICAPARC'] ??
            NovoCodigo::geraCodigo($nrOrg, 'COMUNICAPARC', 1, 10, true, false);

        $comuRow['NRPARCNEGOCIO']       = $row['NRPARCNEGOCIO'] ?? null;
        $comuRow['CDFORMACOMU']         = $row['CDFORMACOMU'] ?? null;
        $comuRow['NRENDERECOPARC']      = $row['NRENDERECOPARC'] ?? null;
        $comuRow['CDPREFIXCOMUPARC']    = $row['CDPREFIXCOMUPARC'] ?? null;
        $comuRow['CDCOMUNICAPARC']      = $row['CDCOMUNICAPARC'] ?? null;
        $comuRow['NRORG']               = $row['NRORG'] ?? $nrOrg;
        $comuRow['IDATIVO']             = $row['IDATIVO'] ?? 'S';
        $comuRow['DTINCLUSAO']          = $row['DTINCLUSAO'] ?? date('d/m/Y');
        $comuRow['DTULTATU']            = $row['DTULTATU'] ?? null;
        $comuRow['NRORGINCLUSAO']       = $row['NRORGINCLUSAO'] ?? $nrOrg;
        $comuRow['CDOPERINCLUSAO']      = $row['CDOPERINCLUSAO'] ?? $cdOperador;
        $comuRow['NRORGULTATU']         = $row['NRORGULTATU'] ?? null;
        $comuRow['CDOPERULTATU']        = $row['CDOPERULTATU'] ?? null;
        $comuRow['CDCOMPLECOMUPARC']    = $row['CDCOMPLECOMUPARC'] ?? null;
        $comuRow['DSCONTATO']           = $row['DSCONTATO'] ?? null;

        

        $this->parceiroNegocioRepository->setComunicaParc($comuRow);

        return $comuRow['NRCOMUNICAPARC'];
    }
    
    public function getParcNegocioTodos(
        string $nrorg,
        string $dtmescompetenc = null,
        string $existeVinculos = null
    ) {
        $dtmescompetenc = !empty($dtmescompetenc) ? $dtmescompetenc : $this->periodoRepository->getDtmescompetencByPeriodo($nrorg);
        return $this->parceiroNegocioRepository->getParcNegocioTodos($nrorg, $dtmescompetenc, $existeVinculos);
    }

    public function getEnderecoParcByTipo(
        int $nrParcNegocio,
        string $cdTipoEndereco
    ) {
        $nrOrg = (int) $this->environment->getNrOrg();

        $enderecoParc = $this->parceiroNegocioRepository->getEnderecoParcByTipo(
            $nrOrg,
            $nrParcNegocio,
            $cdTipoEndereco
        );

        return $enderecoParc;
    }

    public function deleteEnderecoParc(
        int $nrEnderecoParc
    ) {
        $nrOrg = (int) $this->environment->getNrOrg();

        $this->parceiroNegocioRepository->deleteEnderecoParc(
            $nrOrg,
            $nrEnderecoParc
        );
    }

    public function setEnderecoParcAnterior(
        int $nrEnderecoParc
    ) {
        $nrOrg = (int) $this->environment->getNrOrg();

        $this->parceiroNegocioRepository->setEnderecoParcAnterior(
            $nrOrg,
            $nrEnderecoParc
        );
    }
}